package org.springblade.aop;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.report.entity.ReportDatasource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import lombok.extern.slf4j.Slf4j;

/**
 * 数据源切面---执行数据源相关的sql时先判断是否有该数据源
 */
@Aspect
@Component
@Slf4j
public class DsAspect {

    @Autowired
    private DynamicDataSourceSupport dynamicDataSourceSupport;

    private void addDs(ReportDatasource ds) {
        boolean dsExists = dynamicDataSourceSupport.exists(ds.getPoolName());
        log.info("检测数据源是否存在｜poolname={},dsExists={}", ds.getPoolName(), dsExists);
        if (!dsExists) {
            dynamicDataSourceSupport.addDs(ds);
        }
    }

    @Before("execution(* org.springblade.cube.service.DynamicTableService..*(..))")
    public void beforeMethod(JoinPoint joinPoint) {
        Object[] args = joinPoint.getArgs();
        for (Object arg : args) {
            if (arg instanceof ReportDatasource) {
                ReportDatasource ds = (ReportDatasource) arg;
                addDs(ds);
                break;
            }
        }
    }


}
